// cvisionDlg.cpp : implementation file
//

#include <vector>
#include "stdafx.h"
#include "cvision.h"
#include "cvisionDlg.h"
#include "cvapp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCvisionDlg dialog

CCvisionDlg::CCvisionDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCvisionDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCvisionDlg)
	m_keyf = 0;
	m_pf = 0;
	m_quality = 0.0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CCvisionDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCvisionDlg)
	DDX_Control(pDX, IDC_LIST1, m_list);
	DDX_Text(pDX, IDC_EDIT1, m_keyf);
	DDX_Text(pDX, IDC_EDIT2, m_pf);
	DDX_Text(pDX, IDC_EDIT3, m_quality);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CCvisionDlg, CDialog)
	//{{AFX_MSG_MAP(CCvisionDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON1, OnOpen)
	ON_BN_CLICKED(IDC_BUTTON2, OnProcess)
	ON_BN_CLICKED(IDC_BUTTON4, OnStop)
	ON_BN_CLICKED(IDC_BUTTON3, OnSave)
	ON_LBN_DBLCLK(IDC_LIST1, OnDblclkList1)
	ON_LBN_SELCHANGE(IDC_LIST1, OnSelchangeList1)
	ON_BN_CLICKED(IDC_BUTTON5, OnCamera)
	ON_BN_CLICKED(IDC_BUTTON6, OnCompress)
	ON_BN_CLICKED(IDC_BUTTON7, OnCamSet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCvisionDlg message handlers

BOOL CCvisionDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
    procseq= new SequenceProcessor();
    modified= TRUE;
    camera= FALSE;

	m_keyf= m_pf= -1;
	m_quality= -1.;
    UpdateData(false);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CCvisionDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CCvisionDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CCvisionDlg::OnOpen() 
{
  CFileDialog dlg(TRUE, _T("*.bmp"), "", 
                  OFN_FILEMUSTEXIST|OFN_PATHMUSTEXIST|OFN_HIDEREADONLY,
                  "AVI files (*.avi) |*.avi|All Files (*.*)|*.*||",NULL);

  char title[]= {"Open Sequence"};
  dlg.m_ofn.lpstrTitle= title;
  camera= FALSE;

  if (dlg.DoModal() == IDOK) {

    CString path= dlg.GetPathName();

    procseq->setInput(path);
    modified= TRUE;
  }
}

void CCvisionDlg::OnProcess() 
{

  if (modified) {

	procseq->stop();
    procseq->clear();
    procseq->createFilterGraph();
  }

  camera= modified= FALSE;

  procseq->getCompressorOptions(m_keyf, m_pf, m_quality);
  UpdateData(false);

  std::vector<CString> names= procseq->enumFilters();

  m_list.ResetContent();
  for (int i=0; i<names.size(); i++)
    m_list.AddString(names[i]);

  procseq->run();
}

void CCvisionDlg::OnOK() 
{
    procseq->stop();
    if (procseq != 0)
      delete procseq;

    CDialog::OnOK();
}

void CCvisionDlg::OnCancel() 
{
	
    procseq->stop();
    if (procseq != 0)
      delete procseq;

    CDialog::OnCancel();
}

void CCvisionDlg::OnStop() 
{
    procseq->stop();
}

void CCvisionDlg::OnSave() 
{
  CFileDialog dlg(FALSE, _T(".avi"), "", 
                  OFN_OVERWRITEPROMPT|OFN_HIDEREADONLY,
                  "avi files (*.avi) |*.avi|All Files (*.*)|*.*||",NULL);

  char title[]= {"Save sequence"};
  dlg.m_ofn.lpstrTitle= title;

  if (dlg.DoModal() == IDOK) {

    CString path= dlg.GetPathName();

    procseq->setOutput(path);
    modified= TRUE;
  }			

  std::vector<CString> fname;
  std::vector<CLSID> fclsid;
			
  camera= FALSE;

  enumFilters(CLSID_VideoCompressorCategory, fname, fclsid);

  m_list.ResetContent();
  for (int i=0; i<fname.size(); i++)
    m_list.AddString(fname[i]);
}

void CCvisionDlg::OnDblclkList1() 
{
	CString txt;
	m_list.GetText(m_list.GetCurSel(),txt);
    UpdateData(true);
	if (camera) {
  	  procseq->setCamera(txt);
	} else {
	  procseq->setCompressor(txt, m_keyf, m_pf, m_quality);
	}
}

void CCvisionDlg::OnSelchangeList1() 
{
	CString txt;
	m_list.GetText(m_list.GetCurSel(),txt);
    UpdateData(true);

	if (camera) {
  		procseq->setCamera(txt);
	} else {
		procseq->setCompressor(txt, m_keyf, m_pf, m_quality);
	}
}

void CCvisionDlg::OnCamera() 
{
  camera= TRUE; 
  modified= TRUE;

  std::vector<CString> fname;
  std::vector<CLSID> fclsid;
			
  enumFilters(CLSID_VideoInputDeviceCategory, fname, fclsid);

  m_list.ResetContent();
  for (int i=0; i<fname.size(); i++)
    m_list.AddString(fname[i]);	
}

void CCvisionDlg::OnCompress() 
{
  modified= TRUE;

  std::vector<CString> fname;
  std::vector<CLSID> fclsid;
			
  camera= FALSE;

  enumFilters(CLSID_VideoCompressorCategory, fname, fclsid);

  m_list.ResetContent();
  for (int i=0; i<fname.size(); i++)
    m_list.AddString(fname[i]);
	
}

void CCvisionDlg::OnCamSet() 
{

  if (modified) {

	procseq->stop();
    procseq->clear();
    procseq->createFilterGraph();
  }

  procseq->setCameraOptions(*this);
}
